/*****************************************************************************

Xprotolab - AVR XMEGA Oscilloscope and Development Kit

Gabotronics C.A.
March 2011

www.gabotronics.com
email me at: gabriel@gabotronics.com

*****************************************************************************/

#include <avr/io.h>
#include <util/delay.h>
#include "main.h"
#include "mso.h"
#include "logic.h"
#include "SSD1306.h"
#include "ex4bit.h"

#define SOH     0x01    // start of heading
#define STX     0x02    // start of text
#define EOT     0x04    // end of transmission
#define ACK     0x06    // acknowledge
#define NAK     0x15    // negative acknowledge
#define CAN     0x18    // cancel

// Monochorome 128x64 bitmap header
uint8_t BMP[62] PROGMEM = {
    0x42,0x4D,0x3E,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x3E,0x00,0x00,0x00,0x28,0x00,
    0x00,0x00,0x80,0x00,0x00,0x00,0x40,0x00,0x00,0x00,0x01,0x00,0x01,0x00,0x00,0x00,
    0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0xff,0xff,0x00 };

void SendBMP(void);

void send(uint8_t tx);
uint8_t read(void);

void interface(void) {
    char command;
    uint8_t *p;
    uint8_t i;
   // while(USARTE0.STATUS,USART_RXCIF_bp) {
        command = USARTE0.DATA;
        switch(command) {
            case '0': menu=0; key=KA; break;    // Stop
            case '+': menu=0; key=KB; break;    // Sampling Rate ++
            case '-': menu=0; key=KC; break;    // Sampling Rate --
            case '*': menu=0; key=KBC; break;   // Autosetup
            case '1': menu=1; key=KA; break;    // CH1 on/off
            case '4': menu=1; key=KB; break;    // CH1 gain--
            case '7': menu=1; key=KC; break;    // CH1 gain++
            case '2': menu=2; key=KA; break;    // CH2 on/off
            case '5': menu=2; key=KB; break;    // CH2 gain--
            case '8': menu=2; key=KC; break;    // CH2 gain++
            case '3': menu=3; key=KA; break;    // CHD on/off
            case 'a':   // Send firmware version
                p = VERSION+8;
                while(pgm_read_byte(p) != 0x00) send(pgm_read_byte(p++));
                send(10);
            break;
            case 'r':   // Send CH1 data
                i=0;
                do {
                    send(CH1.data[i]);
                } while(++i);
            break;
            case 's':   // Send CH2 data
                i=0;
                do {
                    send(CH2.data[i]);
                } while(++i);
            break;
            case 't':   // Send CHD data
                i=0;
                do {
                    send(CHD.data[i]);
                } while(++i);
            break;
            case 'u':   // Send settings
                p=&CH1; for(i=0; i<4; i++)  send(*p++); // CH1
                p=&CH2; for(i=0; i<4; i++)  send(*p++); // CH2
                p=&CHD; for(i=0; i<6; i++)  send(*p++); // CHD
                p=0;    for(i=0; i<12; i++) send(*p++); // GPIO
            break;
            case 'v':   // Read settings
                if(read()=='w') {   // protection
                    p=&CH1; for(i=0; i<4; i++)  *p++=read(); // CH1
                    p=&CH2; for(i=0; i<4; i++)  *p++=read(); // CH2
                    p=&CHD; for(i=0; i<6; i++)  *p++=read(); // CHD
                    p=0;    for(i=0; i<12; i++) *p++=read(); // GPIO
                    // Also use checksum later
                }
            break;
            case 'C':   // Send BMP
                SendBMP();
            break;
            default: USARTE0.DATA = command;   // echo
        }
   // }
    if(key) {
        setbit(Signals, update);         // Valid key
        setbit(Signals, userinput);
    }
    RTC.CNT=0;  // Clear screen saver timer
}

void send(uint8_t tx) {
    while ( !testbit(USARTE0.STATUS,USART_DREIF_bp) ) ;  // wait until ready to transmit
    USARTE0.DATA = tx;
}

uint8_t read(void) {
    uint8_t timeout=100;
    while((timeout--) || !testbit(USARTE0.STATUS,USART_RXCIF_bp)) {
        _delay_ms(1);
    }
    return USARTE0.DATA;
}


/*
// Waits for a character from the serial port for a certain time, 
// If no character is received in 5 seconds, it returns 0
char get_timeout(void) {
    uint8_t time_out=0,t;
    char c;
    do {
        time_out++; if(time_out>100) return 0;
        _delay_ms(50);
        t=uart_test();
    } while (t==0);
    c = uart_get();
    send(c);
    return c;
}*/

// Test pixel on display buffer
uint8_t test_pixel(uint8_t x, uint8_t y) {
    if(display_buffer[((uint16_t)(y<<4)&0xFF80) + x] & (uint8_t)(0x01 << (y & 0x07))) return 0;
    else return 1;
}

uint8_t transpose(uint8_t x, uint8_t y) {
    uint8_t data=0;
    data  = test_pixel(x+0,y); data=data<<1;
    data |= test_pixel(x+1,y); data=data<<1;
    data |= test_pixel(x+2,y); data=data<<1;
    data |= test_pixel(x+3,y); data=data<<1;
    data |= test_pixel(x+4,y); data=data<<1;
    data |= test_pixel(x+5,y); data=data<<1;
    data |= test_pixel(x+6,y); data=data<<1;
    data |= test_pixel(x+7,y);
    return data;
}

// Send data from LCD to PC via RS-232 using the XMODEM protocol
void SendBMP(void) {
    uint8_t rx,data,Packet=1, n=0,i,j,time_out=0;
    CRC16_High=0;
    CRC16_Low=0;

    // First Block
    send(SOH);
    send(Packet);       // Send Packet number
    send(255-Packet);   // Send Packet number 1's complement
    // Send BMP Header
    for(i=0; i<62; i++) {
        n++;
        data = pgm_read_byte_near(BMP+i);
        CRC16_Update(data);
        send(data);
    }

    // Send LCD data, 64 lines
    for(i=63; i!=255; i--) {
        // Each LCD line consists of 16 bytes
        for(j=0; j<16; j++) {
            data=transpose(j*8,i);
            CRC16_Update(data);
            send(data);
            n++;
            if(n==128)  {   // end of 128byte block
                n=0;
                send(CRC16_High);
                send(CRC16_Low);
                time_out=0;
                // Wait for ACK
                while (!testbit(USARTE0.STATUS,USART_RXCIF_bp)) {
                    time_out++; if(time_out>200) return;
                    _delay_ms(1);
                }
                rx = read();
                if(rx!=ACK) return; // Error -> cancel transmission
                Packet++;
                send(SOH);
                send(Packet);
                send(255-Packet);
                CRC16_High=0;
                CRC16_Low=0;
            }
        }
    }

    // End of last block
    for(; n<128; n++) { // send remainder of block
        data= 0x1A;     // pad with 0x1A which marks the end of file
        CRC16_Update(data);
        send(data);
    }
    send(CRC16_High);
    send(CRC16_Low);
    time_out=0;
    while (!testbit(USARTE0.STATUS,USART_RXCIF_bp)) {
        time_out++; if(time_out>200) return;
        _delay_ms(1);
    }
    rx = read();
    if(rx!=ACK) return;

    send(EOT);
    time_out=0;
    while (!testbit(USARTE0.STATUS,USART_RXCIF_bp)) {
        time_out++; if(time_out>200) return;
        _delay_ms(1);
    }
    rx = read();
    if(rx!=NAK) return;
    send(EOT);
    time_out=0;
    while (!testbit(USARTE0.STATUS,USART_RXCIF_bp)) {
        time_out++; if(time_out>200) return;
        _delay_ms(1);
    }
    rx = read();
    if(rx!=ACK) return;
}
