/****************************************************************************

Xprotolab - AVR XMEGA Oscilloscope and Development Kit

Gabotronics C.A.
July 2010

Copyright 2010 Gabriel Anzziani

This program is distributed under the terms of the GNU General Public License 

www.gabotronics.com
email me at: gabriel@gabotronics.com

*****************************************************************************/

#ifndef _LOGIC_H
#define _LOGIC_H

typedef struct {
    uint8_t position;           // Channel position
    uint8_t mask;               // Mask inputs
    union {
        uint8_t option;         // All options in a byte
        struct {
            uint8_t on:1;       // Channel on
            uint8_t invert:1;   // Invert channel
            uint8_t low:1;      // Thick line when logic '0'
            uint8_t pull:1;     // Pull input
            uint8_t up:1;       // Pull up / down
        } ctrl;
    };
    union {
        uint8_t param;          // Decode parameters
        struct {
            uint8_t CPOL:1;     // SPI Clock Polarity
            uint8_t CPHA:1;     // SPI Clock Phase
            uint8_t parmode:1;  // Use UART parity
            uint8_t parity:1;   // UART parity odd or even
            uint8_t stopbit:1;  // UART 1 or 2 stop bits
            uint8_t baud:3;     // UART Baud rate
        } ctrl2;
    };
    union {
        uint8_t decode;         // Decode
        struct {
            uint8_t hexs:1;     // Serial Hex Display
            uint8_t hexp:1;     // Parallel Hex Display
            uint8_t i2c:1;      // I2C decoding
            uint8_t rs232:1;    // RS232 decoding
            uint8_t spi:1;      // SPI decoding
            uint8_t ascii:1;    // Use ASCII with RS232
        } ctrl3;
    };
    uint8_t data[256];          // Samples
} DCHANNEL;

extern DCHANNEL CHD;            // Digital Channel

void HEXSerial(void);           // Display the HEX value of the digital stream
void Sniffer(void);             // Protocol Sniffer

#endif
