/****************************************************************************

Xprotolab - AVR XMEGA Oscilloscope and Development Kit

Gabotronics C.A.
December 2010

Copyright 2010 Gabriel Anzziani

This program is distributed under the terms of the GNU General Public License 

www.gabotronics.com
email me at: gabriel@gabotronics.com

*****************************************************************************/

#ifndef _MAIN_H
#define _MAIN_H

#include <stdint.h>
#include <avr/eeprom.h>
#include "mygccdef.h"
#include "asmutil.h"
#include "data.h"
#include "SSD1306.h"

#define NOKEY   0
#define KA      1       // Cancel, back
#define KB      2       // Decrease, left
#define KC      4       // Plus, right
#define KBC     6       // Auto setup
#define KD      8       // Menu, Ok, Power
#define KBR     10      // Repeat KB
#define KCR     11      // Repeat KC

// PORT DEFINITIONS
#define POWER       0   // PORTC
#define ONBUTTON    1   // PORTC
#define HEATER      2   // PORTC
#define BACKLITE    3   // PORTE

#define TGLGRN()    PORTC.OUTTGL = 0x10

#define TGLBKLT()   togglebit(VPORT0.OUT, BACKLITE)
#define ONBKLT()    clrbit(VPORT1.OUT, BACKLITE)
#define OFFBKLT()   setbit(VPORT1.OUT, BACKLITE)
#define ONHEAT()    setbit(VPORT2.OUT, HEATER)
#define OFFHEAT()   clrbit(VPORT2.OUT, HEATER)

#define SDWP		2	// Write protect switch (PF2)
#define SDCD		1	// Card detect switch (PE1)

// Global variables, using GPIO for optimized access
#define Signals GPIO0   // Signals
#define MStatus GPIO1   // MSO Status
#define Mcursor GPIO2   // MSO Cursors
#define MFFT    GPIO3   // MSO FFT Options
#define Mset    GPIO4   // MSO settings
#define srate   GPIO5   // Sampling rate
#define tpre    GPIO6   // Number of Pre Trigger samples
#define tdelay  GPIO7   // Trigger delay
#define tsource GPIO8   // Trigger source
#define HPos    GPIO9   // Horizontal position
#define display GPIOA   // Display options
#define tlevel  GPIOB   // Trigger Level
#define keyrep  GPIOC   // Key repeat
#define key     GPIOD   // Current key pressed
#define index   GPIOE   // sample index
#define menu    GPIOF   // Menu

// Signal bits (GPIO0)
#define update      0       // Update
#define keydown     1       // A key is being pressed
#define redraw      2       // Redraw screen
#define userinput   3       // User input received
#define nextsamp    4       // Get next sample (used in slow sampling rates)

#define serialin    6       // Received command from serial port
#define acquired    7       // Data has been acquired (for slow sampling)

uint8_t prandom(void);
void sound(uint8_t tone);
int8_t Sin(uint8_t angle);
int8_t Cos(uint8_t angle);
uint8_t addwsat(uint8_t a, int8_t b);
char NibbleToChar(uint8_t nibble);  // Converts a nibble to the corresponding ASCII representing the HEX value
void printhex(uint8_t n);           // Prints a HEX number
uint8_t half(uint8_t number);
void InitDMA(void);

void CCPWrite( volatile uint8_t * address, uint8_t value );

extern uint8_t  Letter[17];
extern uint8_t  AWGBuffer[256];
extern uint16_t tempCal;
extern uint16_t tempavrg;
extern uint16_t Vin;            // Input voltage
extern uint16_t Vinavrg;        // Input voltage averaged
extern uint8_t bigtemp[1280];   
extern uint8_t EEMEM EESleepTime;     // Sleep timeout in minutes

void MSO(void);

#endif
