/****************************************************************************

Xprotolab - AVR XMEGA Oscilloscope and Development Kit

Gabotronics C.A.
December 2010

Copyright 2010 Gabriel Anzziani

This program is distributed under the terms of the GNU General Public License 

www.gabotronics.com
email me at: gabriel@gabotronics.com

*****************************************************************************/

#ifndef _MSO_H
#define _MSO_H

#include <stdint.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include "ffft.h"

// MStatus bits     (GPIO1)
#define freetrig    0       // Free trigger
#define normal      1       // Normal trigger
#define single      2       // Single trigger
#define autotrg     3       // Auto trigger
#define trigger     4       // oscilloscope triggered
#define stop        5       // Scope stopped
#define trigdir     6       // trigger falling or rising
#define acquired    7       // Data has been acquired

// Mcursor bits     (GPIO2)
#define reference   0       // Reference waveforms on
#define cursorh1    1       // CH1 Horizontal Cursor on
#define cursorh2    2       // CH2 Horizontal Cursor on
#define cursorv     3       // Vertical Cursor on
#define lockcur     4       // Lock Automatic Cursors
#define rotv        5       // Rotary encoder on vertical cursor
#define roth        6       // Rotary encoder on horizontal cursor ch1 or ch2 (if not rotv)
#define adjusting   7       // Auto setup in process

// MFFT bits        (GPIO3)
#define hamming     0       // Hamming window
#define hann        1       // Hann window
#define cosine      2       // Cosine window
#define triangle    3       // Triangular window
#define uselog      4       // Apply logarithm to FFT
#define vdc         5       // Calculate VDC
#define vp_p        6       // Calculate VPP
#define autoset     7       // Autoset MSO

// Mset bits        (GPIO4)
#define awgsweep    0       // AWG sweep
#define persistent  1       // Persistent Display
#define line        2       // Continous Drawing
#define showset     3       // Show scope settings ( time/div volts/div)
#define scope       4       // MSO mode
#define meter       5       // Multimeter mode
#define fft         6       // FFT mode
#define xymode      7       // X-Y mode

// display bits     (GPIOA) // Display options
//      grid        0       // Grid settings (2 bits)
//      grid        1
#define disp_inv    6       // Invert Display
#define flip        7       // Flip Display

typedef struct {
    int8_t  position;           // Channel position
    union {
        uint8_t option;         // All options in a byte
        struct {
            uint8_t on:1;       // Channel on
            uint8_t x10:1;      // x10 probe
            uint8_t x100:1;     // x100 probe
            uint8_t x1000:1;    // x1000 probe
            uint8_t invert:1;   // Invert channel
            uint8_t average:1;  // Average samples
            uint8_t math:1;     // math (CH1+CH2 or CH2*CH2 active)
            uint8_t addmult:1;  // CH1+CH2 or CH2*CH2
        } ctrl;
    };
    uint8_t gain;               // Channel gain
    int8_t  offset;             // Channel offset
    uint8_t max;                // Maximum value
    uint8_t min;                // Minimum value
    uint8_t vpp;                // Peak to peak
    uint8_t data[256];          // Samples
    uint8_t f;                  // Maximum frequency
} ACHANNEL;

extern ACHANNEL CH1, CH2;       // Analog Channel 1, Channel 2

// Function prototypes
void ShowCursorV(void);                 // Display Vertical Cursor
// Display Horizontal Cursor
void ShowCursorH(uint8_t gain, uint8_t HcursorA, uint8_t HcursorB, int8_t CHPos);
void MenuHelp(void);                    // Display Keypad Menu Help
void LoadEE(void);                      // Load settings from EEPROM
void SaveEE(void);                      // Save settings to EEPROM
void Apply(void);                       // Apply oscilloscope settings
void printhex(uint8_t n);               // Prints a HEX number

extern signed char EEMEM offsetsCH1[8];
extern signed char EEMEM offsetsCH2[8];
extern uint8_t  VcursorA;               // Vertical cursor A
extern uint8_t  VcursorB;               // Vertical cursor B

#endif
